<?php
/*
 * tweeper - a Twitter to RSS web scraper
 * 
 * Copyright (C) 2013  Antonio Ospite <ospite@studenti.unina.it>
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

date_default_timezone_set('UTC');

define('USER_AGENT', 'Mozilla/5.0 (Windows NT 6.1; rv:22.0) Gecko/20130405 Firefox/22.0');

function str_to_gmdate($date)
{
  $timestamp = strtotime($date);
  return epoch_to_gmdate($timestamp);
}

function epoch_to_gmdate($timestamp)
{
  return gmdate('D, d M Y H:i:s', $timestamp) . ' GMT';
}

function get_url_info($url)
{
  $ch = curl_init($url);
  curl_setopt_array($ch, array(
      CURLOPT_HEADER => TRUE,
      CURLOPT_NOBODY => TRUE,
      CURLOPT_FOLLOWLOCATION => TRUE,  // follow http redirects to get the real URL
      CURLOPT_RETURNTRANSFER => TRUE,
      CURLOPT_SSL_VERIFYHOST => FALSE,
      CURLOPT_SSL_VERIFYPEER => FALSE,
      CURLOPT_USERAGENT => USER_AGENT,
  ));
  curl_exec($ch);
  $url_info = curl_getinfo($ch);;
  curl_close($ch);

  return $url_info;
}

function generate_enclosure($url)
{
  $supported_content_types = array(
    "application/ogg",
    "audio/aac",
    "audio/mp4",
    "audio/mpeg",
    "audio/vorbis",
    "audio/wav",
    "audio/webm",
    "audio/x-midi",
    "image/gif",
    "image/jpeg",
    "video/avi",
    "video/mp4",
    "video/mpeg",
    "video/ogg",
  );

  $url_info = get_url_info($url);

  $supported = in_array($url_info['content_type'], $supported_content_types);
  if (!$supported) {
    error_log("Unsupported enclosure content type \"" . $url_info['content_type'] . "\" for URL: " . $url_info['url']);
    return '';
  }

  $dom = new DomDocument();
  $enc = $dom->createElement('enclosure');
  $enc->setAttribute('url', $url_info['url']);
  $enc->setAttribute('length', $url_info['download_content_length']);
  $enc->setAttribute('type', $url_info['content_type']);

  $dom->appendChild($enc);

  return $dom->saveXML($enc);
}


class Tweeper {

  public function __construct($stylesheet, $generate_enclosure = FALSE) {
    $stylesheet_contents = $this->get_contents($stylesheet);

    $xslDoc = new DOMDocument();
    $xslDoc->loadXML($stylesheet_contents);

    $this->xsltProcessor = new XSLTProcessor();
    $this->xsltProcessor->registerPHPFunctions();
    $this->xsltProcessor->setParameter('', 'generateEnclosure', $generate_enclosure);
    $this->xsltProcessor->importStylesheet($xslDoc);
  }

  private static function get_contents($uri) {
    # https://www.wjsams.com/c/docs/Wiki/Php-HowToSetUserAgentOnFileGetContents
    $opts = array(
      'http' => array(
        'method' => "GET",
        'header' => join(array(
          "Accept-language: en\r\n",
          "User-Agent: {" + USER_AGENT + "}\r\n"
        ))
      )
    );

    $context = stream_context_create($opts);
    $contents = file_get_contents($uri, false, $context);
    return $contents;
  }

  public function tweep($uri) {
    $html = Tweeper::get_contents($uri);

    $xmlDoc = new DOMDocument();
    $xmlDoc->loadHTML($html);

    $output = $this->xsltProcessor->transformToXML($xmlDoc);

    if (FALSE === $output) {
      trigger_error('XSL transformation failed.', E_USER_ERROR);
      return NULL;
    }
    return $output;
  }
}

function usage($argv)
{
  if (php_sapi_name() != 'cli')
    $usage = htmlentities("{$_SERVER['SCRIPT_NAME']}?src_url=<src_url>&generate_enclosure=<0|1>");
  else
    $usage = "{$argv[0]} [-e|-h|--help] <src_url>\n";

  return "usage: $usage";
}

function parse_options_cli($argv, $argc)
{
  $options = array(
    'generate_enclosure' => FALSE
  );

  if ($argc < 2)
    return $options;

  $cli_options = getopt("eh", array("help"));
  foreach ($cli_options as $opt => $val) {
    switch ($opt) {
    case 'e':
      $options['generate_enclosure'] = TRUE;
      break;
    case 'h':
    case 'help':
      echo usage($argv);
      exit(0);
    default:
      die(usage($argv));
    }
  }

  $options['src_url'] = $argv[count($cli_options) + 1];

  return $options;
}

function parse_options_query_string()
{
  $options = array(
    'generate_enclosure' => FALSE
  );

  if (isset($_GET['src_url']))
    $options['src_url'] = $_GET['src_url'];

  if (isset($_GET['generate_enclosure']))
    $options['generate_enclosure'] = $_GET['generate_enclosure'] == 1;

  return $options;
}


if (php_sapi_name() != 'cli')
  $options = parse_options_query_string();
else
  $options = parse_options_cli($argv, $argc);

if (!isset($options['src_url']))
  die(usage($argv));

$url = parse_url($options['src_url']);
if (FALSE === $url || empty($url["host"]))
  die("Invalid url: ${options['src_url']}\n");

$stylesheet = __DIR__ . "/rss_converter_" . $url["host"] . ".xsl";
if (FALSE === file_exists($stylesheet))
  die("Conversion to RSS not supported: {$url["host"]}\n");

$tweeper = new Tweeper($stylesheet, $options['generate_enclosure']);
echo $tweeper->tweep($options['src_url']);
